﻿

//Pocklington - Diemitko
//https://studfile.net/preview/5367553/page:45/
//https://en.wikipedia.org/wiki/Pocklington_primality_test


using System.Diagnostics;
using exc.jdbi.DiemitkoTheorem;
using exc.jdbi.VeryBigPrimes.Generators;


namespace exc.jdbi.DiemitkoTheoremTest;


public class Program
{

  public static void Main()
  {

    ////Einfache Testklasse
    //DiemitkoPrimeGeneratorTest.
    //TestDiemitkoPrimeGenerator();

    var count = 5;
    DiemitkoBigPrime512Test(count, false);
    DiemitkoBigPrime1024Test(count, false);
    DiemitkoBigPrime2048Test(count, false);


    Console.WriteLine();
    Console.WriteLine("FINISH");
    Console.ReadLine();
  }

  private static void DiemitkoBigPrime512Test(int rounds, bool checkmrprime = false)
  {
    Console.WriteLine($"***********************");
    Console.WriteLine($"{nameof(DiemitkoBigPrime512Test)}");
    Console.WriteLine($"***********************");
    Console.WriteLine();

    var bitslength = BitsPrime.Bit512;
    var with_mr_prime_check = checkmrprime;
    var pcount = DiemitkoPrimeGenerator.PARALLEL_COUNT;
    var bitshalflength = (BitsPrime)((int)bitslength / 2);

    var time = 0L;
    var sw = new Stopwatch();
    for (int i = 0; i < rounds; i++)
    {
      var halfprime = BigPrimeGenerator.RngBigPrimes(bitshalflength, 1)[0];

      sw.Start();
      using var result = DiemitkoPrimeGenerator.StartDiemitkoSingleAsync(halfprime, pcount);
      sw.Stop();

      var prime = result.Result;

      if (with_mr_prime_check && !DiemitkoPrimeGenerator.IsMRPrime(prime))
        throw new ArgumentException($"{prime}");

      Console.WriteLine(prime.ToString());

      var digits = prime.ToString().Length;
      var nob = DiemitkoPrimeGenerator.ToNumberOfBits(prime);
      Console.WriteLine($"count = 1; bits = {nob}; digits = {digits}; time = {sw.ElapsedMilliseconds - time} ms");
      Console.WriteLine();

      time = sw.ElapsedMilliseconds;
    }

    Console.WriteLine($"{nameof(DiemitkoBigPrime512Test)} total time = {sw.ElapsedMilliseconds} ms");
    Console.WriteLine(); Console.WriteLine();
  }

  private static void DiemitkoBigPrime1024Test(int rounds, bool checkmrprime = false)
  {
    Console.WriteLine($"************************");
    Console.WriteLine($"{nameof(DiemitkoBigPrime1024Test)}");
    Console.WriteLine($"************************");
    Console.WriteLine();

    var bitslength = BitsPrime.Bit1024;
    var with_mr_prime_check = checkmrprime;
    var pcount = DiemitkoPrimeGenerator.PARALLEL_COUNT;
    var bitshalflength = (BitsPrime)((int)bitslength / 2);

    var time = 0L;
    var sw = new Stopwatch();
    for (int i = 0; i < rounds; i++)
    {
      var halfprime = BigPrimeGenerator.RngBigPrimes(bitshalflength, 1)[0];

      sw.Start();
      using var result = DiemitkoPrimeGenerator.StartDiemitkoSingleAsync(halfprime, pcount);
      sw.Stop();

      var prime = result.Result;

      if (with_mr_prime_check && !DiemitkoPrimeGenerator.IsMRPrime(prime))
        throw new ArgumentException($"{prime}");

      Console.WriteLine(prime.ToString());

      var digits = prime.ToString().Length;
      var nob = DiemitkoPrimeGenerator.ToNumberOfBits(prime);
      Console.WriteLine($"count = 1; bits = {nob}; digits = {digits}; time = {sw.ElapsedMilliseconds - time} ms");
      Console.WriteLine();

      time = sw.ElapsedMilliseconds;
    }

    Console.WriteLine($"{nameof(DiemitkoBigPrime1024Test)} total time = {sw.ElapsedMilliseconds} ms");
    Console.WriteLine(); Console.WriteLine();
  }

  private static void DiemitkoBigPrime2048Test(int rounds, bool checkmrprime = false)
  {
    Console.WriteLine($"************************");
    Console.WriteLine($"{nameof(DiemitkoBigPrime2048Test)}");
    Console.WriteLine($"************************");
    Console.WriteLine();

    var bitslength = BitsPrime.Bit2048;
    var with_mr_prime_check = checkmrprime;
    var pcount = DiemitkoPrimeGenerator.PARALLEL_COUNT;
    var bitshalflength = (BitsPrime)((int)bitslength / 2);

    var time = 0L;
    var sw = new Stopwatch();
    for (int i = 0; i < rounds; i++) 
    { 
      var halfprime = BigPrimeGenerator.RngBigPrimes(bitshalflength, 1)[0];
      
      sw.Start();
      using var result = DiemitkoPrimeGenerator.StartDiemitkoSingleAsync(halfprime, pcount);
      sw.Stop();

      var prime = result.Result;

      if (with_mr_prime_check && !DiemitkoPrimeGenerator.IsMRPrime(prime))
        throw new ArgumentException($"{prime}");

      Console.WriteLine(prime.ToString());

      var digits = prime.ToString().Length;
      var nob = DiemitkoPrimeGenerator.ToNumberOfBits(prime);
      Console.WriteLine($"count = 1; bits = {nob}; digits = {digits}; time = {sw.ElapsedMilliseconds - time} ms");
      Console.WriteLine();

      time = sw.ElapsedMilliseconds;     
    }

    Console.WriteLine($"{nameof(DiemitkoBigPrime2048Test)} total time = {sw.ElapsedMilliseconds} ms");
    Console.WriteLine(); Console.WriteLine();
  }
}
 